import os
from pilkit.lib import Image


def get_image_file(image_name='reference.png'):
    """
    See also:

    http://en.wikipedia.org/wiki/Lenna
    http://sipi.usc.edu/database/database.php?volume=misc&image=12

    """
    dir = os.path.dirname(__file__)
    path = os.path.join(dir, 'assets', image_name)
    return open(path, 'r+b')

def create_image():
    return Image.open(get_image_file())

def compare_images(a, b):
  if a.size != b.size:
    return False

  rows, cols = a.size

  for row in range(rows):
    for col in range(cols):
      if a.getpixel((row, col)) != b.getpixel((row, col)):
        return False

  return True



import os
import io
import pytest
from unittest.mock import Mock, patch
from tempfile import NamedTemporaryFile

from pilkit.exceptions import UnknownFormat, UnknownExtension
from pilkit.lib import Image
from pilkit.utils import (extension_to_format, format_to_extension, FileWrapper,
                          save_image, prepare_image, quiet)

from .utils import create_image


def test_extension_to_format():
    assert extension_to_format('.jpeg') == 'JPEG'
    assert extension_to_format('.rgba') == 'SGI'


def test_format_to_extension_no_init():
    assert format_to_extension('PNG') == '.png'
    assert format_to_extension('ICO') == '.ico'


def test_unknown_format():
    with pytest.raises(UnknownFormat):
        format_to_extension('TXT')


def test_unknown_extension():
    with pytest.raises(UnknownExtension):
        extension_to_format('.txt')


def test_default_extension():
    """
    Ensure default extensions are honored.

    Since PIL's ``Image.EXTENSION`` lists ``'.jpe'`` before the more common
    JPEG extensions, it would normally be the extension we'd get for that
    format. ``pilkit.utils.DEFAULT_EXTENSIONS`` is our way of specifying which
    extensions we'd prefer, and this tests to make sure it's working.

    """
    assert format_to_extension('JPEG') == '.jpg'


def test_filewrapper():

    class K(object):
        def fileno(self):
            raise io.UnsupportedOperation

    with pytest.raises(AttributeError):
        FileWrapper(K()).fileno()


def test_save_with_filename():
    """
    Test that ``save_image`` accepts filename strings (not just file objects).
    This is a test for GH-8.

    """
    im = create_image()
    with NamedTemporaryFile() as outfile:
        save_image(im, outfile.name, 'JPEG')


def test_format_normalization():
    """
    Make sure formats are normalized by ``prepare_image()``.
    See https://github.com/matthewwithanm/django-imagekit/issues/262
    """
    im = Image.new('RGBA', (100, 100))
    assert 'transparency' in prepare_image(im, 'gIF')[1]


def test_quiet():
    """
    Make sure the ``quiet`` util doesn't error if devnull is unwriteable.
    See https://github.com/matthewwithanm/django-imagekit/issues/294
    """
    mocked = Mock(side_effect=OSError)
    with patch.object(os, 'open', mocked):
        with quiet():
            pass



import os, sys, io, base64
from docutils import nodes
from sphinx.directives.code import CodeBlock


def visit_pil(self, node):
    pass


def depart_pil(self, node):
    pass


class PILNode(nodes.Structural, nodes.Element):
    pass


class PILDirective(CodeBlock):
    has_content = True

    def run(self):
        code_node = CodeBlock.run(self)
        nodes.image()

        from PIL import Image

        lib_path = os.path.abspath('..')
        working_path = os.path.dirname(os.path.realpath(__file__))
        static_path = os.path.join(working_path, '..', '_static')

        sys.path.append(lib_path)
        os.chdir(working_path)

        node = PILNode()

        g = globals()
        l = locals()
        for line in self.content:
            result = exec(line, g, l)

        buffer = io.BytesIO()
        new_img = l.get('new_img')
        new_img.save(buffer, format='PNG')

        image_node = nodes.image()
        image_node['alt'] = 'New Image'
        image_node['uri'] = 'data:image/png;base64,' + base64.b64encode(buffer.getvalue()).decode()

        node += code_node
        node += image_node

        return [node]


def setup(app):
    app.add_node(PILNode, html=(visit_pil, depart_pil))
    app.add_directive('pil-block', PILDirective)

    return {
        'version': '0.1',
        'parallel_read_safe': True,
        'parallel_write_safe': True,
    }



import os
from unittest import mock
import pytest

from pilkit.lib import Image, ImageDraw, ImageColor
from pilkit.processors import (Resize, ResizeToFill, ResizeToFit, SmartCrop,
                               SmartResize, MakeOpaque, ColorOverlay, Convert,
                               GaussianBlur)
from pilkit.processors.resize import Thumbnail
from .utils import create_image, compare_images, get_image_file


def test_smartcrop():
    img = SmartCrop(100, 100).process(create_image())
    assert img.size == (100, 100)


def test_resizetofill():
    img = ResizeToFill(100, 100).process(create_image())
    assert img.size == (100, 100)


def test_resizetofit():
    # First create an image with aspect ratio 2:1...
    img = Image.new('RGB', (200, 100))

    # ...then resize it to fit within a 100x100 canvas.
    img = ResizeToFit(100, 100).process(img)

    # Assert that the image has maintained the aspect ratio.
    assert img.size == (100, 50)


def test_resize_rounding():
    """
    Regression test for matthewwithanm/pilkit#1
    """

    img = Image.new('RGB', (95, 95))
    img = ResizeToFill(28, 28).process(img)
    assert img.size == (28, 28)


def test_resizetofit_mat():
    img = Image.new('RGB', (200, 100))
    img = ResizeToFit(100, 100, mat_color=0x000000).process(img)
    assert img.size == (100, 100)


def test_coloroverlay():
    """
    Test that the ColorOverlay processor
    """
    img = Image.new('RGB', (200, 100))
    color = ImageColor.getrgb('#cc0000')
    img = ColorOverlay(color, overlay_opacity=1.0).process(img)
    assert img.getpixel((0,0)) == (204, 0, 0)

def test_convert():
    img = Image.new('RGBA', (200, 100))

    img_RGBa = Convert("RGBa").process(img)
    assert img_RGBa.mode == "RGBa"

    img_RGBa_RGBA = Convert("RGBA").process(img)
    assert img_RGBa_RGBA.mode == "RGBA"


def test_resize_antialiasing():
    """
    Test that the Resize processor antialiases.

    The Resize processor is used by all of the Resize* variants, so this should
    cover all of resize processors. Basically, this is to test that it converts
    to RGBA mode before resizing.

    Related: jdriscoll/django-imagekit#192

    """
    # Create a palette image and draw a circle into it.
    img = Image.new('P', (500, 500), 1)
    img.putpalette([
        0,   0,   0,
        255, 255, 255,
        0,   0,   255,
    ])
    d = ImageDraw.ImageDraw(img)
    d.ellipse((100, 100, 400, 400), fill=2)

    # Resize the image using the Resize processor
    img = Resize(100, 100).process(img)

    # Count the number of colors
    color_count = len(list(filter(None, img.histogram())))

    assert color_count > 2


def test_upscale():
    """
    Test that the upscale argument works as expected.

    """

    img = Image.new('RGB', (100, 100))

    for P in [Resize, ResizeToFit, ResizeToFill, SmartResize]:
        img2 = P(500, 500, upscale=True).process(img)
        assert img2.size == (500, 500)

        img2 = P(500, 500, upscale=False).process(img)
        assert img2.size == (100, 100)


def test_should_raise_exception_if_anchor_is_passed_and_crop_is_set_to_false():
    with pytest.raises(Exception, match=r"You can't specify an anchor point if crop is False."):
        Thumbnail(height=200, width=200, upscale=False, crop=False, anchor='t')


def test_should_set_crop_to_true_if_anchor_is_passed_without_crop():
    thumb = Thumbnail(height=200, width=200, upscale=False, anchor='t')
    assert thumb.crop


def test_should_raise_exception_when_crop_is_passed_without_height_and_width():
    img = Image.new('RGB', (100, 100))
    with pytest.raises(Exception, match=r"You must provide both a width and height when cropping."):
        Thumbnail(crop=True).process(img)


@mock.patch('pilkit.processors.resize.SmartResize')
def test_should_call_smartresize_when_crop_not_passed(my_mock):
    img = Image.new('RGB', (100, 100))
    Thumbnail(height=200, width=200, upscale=False).process(img)
    assert my_mock.called


@mock.patch('pilkit.processors.resize.SmartResize')
def test_should_repass_upscale_option_true(my_mock):
    img = Image.new('RGB', (100, 100))
    Thumbnail(height=200, width=200, upscale=True).process(img)
    my_mock.assert_called_once_with(width=200, upscale=True, height=200)


@mock.patch('pilkit.processors.resize.SmartResize')
def test_should_repass_upscale_option_false(my_mock):
    img = Image.new('RGB', (100, 100))
    Thumbnail(height=200, width=200, upscale=False).process(img)
    my_mock.assert_called_once_with(width=200, upscale=False, height=200)


@mock.patch('pilkit.processors.resize.ResizeToFill')
def test_should_call_resizetofill_when_crop_and_ancho_is_passed(my_mock):
    img = Image.new('RGB', (100, 100))
    Thumbnail(height=200, width=200, anchor='fake').process(img)
    assert my_mock.called

@mock.patch('pilkit.processors.resize.ResizeToFit')
def test_should_call_resizetofit_when_crop_is_not_passed(my_mock):
    img = Image.new('RGB', (100, 100))
    Thumbnail(height=200, width=200, crop=False).process(img)
    assert my_mock.called

def test_GaussianBlur_radius_3():
    img = GaussianBlur(radius = 3).process(create_image())
    img = img.crop((112,112,144,144))

    expected_img = Image.open(get_image_file("GaussianBlur_radius_3.png"))
    assert compare_images(img, expected_img)

def test_GaussianBlur_radius_7():
    img = GaussianBlur(radius=7).process(create_image())
    img = img.crop((112, 112, 144, 144))

    expected_img = Image.open(get_image_file("GaussianBlur_radius_7.png"))
    assert compare_images(img, expected_img)

def test_make_gifs_opaque():
    dir = os.path.dirname(__file__)
    path = os.path.join(dir, 'assets', 'cat.gif')
    gif = Image.open(path)
    MakeOpaque().process(gif)



#!/usr/bin/env python
import codecs
import os
from setuptools import setup, find_packages


def read(filepath):
    with codecs.open(filepath, 'r', 'utf-8') as f:
        return f.read()


# Load package meta from the pkgmeta module without loading the package.
pkgmeta = {}
pkgmeta_file = os.path.join(os.path.dirname(__file__), 'pilkit', 'pkgmeta.py')
with open(pkgmeta_file) as f:
    code = compile(f.read(), 'pkgmeta.py', 'exec')
    exec(code, pkgmeta)


setup(
    name='pilkit',
    version=pkgmeta['__version__'],
    description='A collection of utilities and processors for the Python Imaging Library.',
    long_description=read(os.path.join(os.path.dirname(__file__), 'README.rst')),
    author='Matthew Tretter',
    author_email='m@tthewwithanm.com',
    license='BSD',
    url='http://github.com/matthewwithanm/pilkit/',
    packages=find_packages(exclude=['tests', 'tests.*']),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'Pillow>=7.0'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Topic :: Utilities'
    ],
)
